@@ -25,6 +25,8 @@
 #include <sys/stat.h>
 #include <sys/fcntl.h>
 
+#include <libecal/e-cal-check-timezones.h>
+
 #include "e-cal-backend-exchange-calendar.h"
 
 #include "e2k-cal-utils.h"
@@ -64,6 +66,8 @@ static gboolean check_owner_partstatus_for_declined (ECalBackendSync *backend,
 gboolean check_for_send_options (icalcomponent *icalcomp, E2kProperties *props);
 static void update_x_properties (ECalBackendExchange *cbex, ECalComponent *comp);
 
+
+
 static void
 add_timezones_from_comp (ECalBackendExchange *cbex, icalcomponent *icalcomp)
 {
@@ -196,6 +200,8 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 	GSList *attachment_list = NULL;
 	gboolean status;
 	ECalBackend *backend = E_CAL_BACKEND (cbex);
+	GError *error = NULL;
+	gboolean retval = TRUE;
 
 	/* Check for attachments */
 	if (uid)
@@ -248,12 +254,20 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 		icalcomponent_free (icalcomp);
 		return status;
 	} else if (kind != ICAL_VCALENDAR_COMPONENT) {
-		icalcomponent_free (icalcomp);
-		if (attachment_list) {
-			g_slist_foreach (attachment_list, (GFunc) g_free, NULL);
-			g_slist_free (attachment_list);
-		}
-		return FALSE;
+		retval = FALSE;
+		goto cleanup;
+	}
+
+	/* map time zones against system time zones and handle conflicting definitions */
+	if (!e_cal_check_timezones (icalcomp,
+				    NULL,
+				    e_cal_backend_exchange_lookup_timezone,
+				    cbex,
+				    &error)) {
+		g_warning ("checking timezones failed: %s", error->message);
+		g_clear_error (&error);
+		retval = FALSE;
+		goto cleanup;
 	}
 
 	add_timezones_from_comp (cbex, icalcomp);
@@ -285,17 +299,20 @@ add_ical (ECalBackendExchange *cbex, const char *href, const char *lastmod,
 		subcomp = icalcomponent_get_next_component (
 				icalcomp, ICAL_VEVENT_COMPONENT);
 	}
+
+ cleanup:
 	icalcomponent_free (icalcomp);
 
 	if (attachment_list) {
 		g_slist_foreach (attachment_list, (GFunc) g_free, NULL);
 		g_slist_free (attachment_list);
 	}
-	return TRUE;
+	return retval;
 }
 
 static const char *event_properties[] = {
 	E2K_PR_CALENDAR_UID,
+	PR_CAL_RECURRING_ID,
 	E2K_PR_DAV_LAST_MODIFIED,
 	E2K_PR_HTTPMAIL_HAS_ATTACHMENT,
 	PR_READ_RECEIPT_REQUESTED,
@@ -319,7 +336,7 @@ get_changed_events (ECalBackendExchange *cbex)
 	E2kRestriction *rn;
 	E2kResultIter *iter;
 	E2kResult *result;
-	const char *prop, *uid, *modtime, *attach_prop, *receipts;
+	const char *prop, *uid, *modtime, *attach_prop, *receipts, *rid;
 	guint status;
 	E2kContext *ctx;
 	int i, status_tracking = EX_NO_RECEIPTS;
@@ -378,6 +395,7 @@ get_changed_events (ECalBackendExchange *cbex)
 			continue;
 		modtime = e2k_properties_get_prop (result->props,
 						   E2K_PR_DAV_LAST_MODIFIED);
+		rid = e2k_properties_get_prop (result->props, PR_CAL_RECURRING_ID);
 
 		attach_prop = e2k_properties_get_prop (result->props,
 						E2K_PR_HTTPMAIL_HAS_ATTACHMENT);
@@ -398,7 +416,7 @@ get_changed_events (ECalBackendExchange *cbex)
 		}
 
 		e_cal_backend_exchange_cache_lock (cbex);
-		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href)) {
+		if (!e_cal_backend_exchange_in_cache (cbex, uid, modtime, result->href, rid)) {
 			g_ptr_array_add (hrefs, g_strdup (result->href));
 			g_hash_table_insert (modtimes, g_strdup (result->href),
 					     g_strdup (modtime));
@@ -559,6 +577,8 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 	GError *error = NULL;
 	ECalBackendExchangeCalendar *cbexc = E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
 
+
+
 	/* Do the generic part */
 	status = E_CAL_BACKEND_SYNC_CLASS (parent_class)->open_sync (
 		backend, cal, only_if_exists, username, password);
@@ -573,8 +593,8 @@ open_calendar (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_Success;
 
 	e_folder_exchange_subscribe (E_CAL_BACKEND_EXCHANGE (backend)->folder,
-                                        E2K_CONTEXT_OBJECT_CHANGED, 30,
-                                        notify_changes, backend);
+			E2K_CONTEXT_OBJECT_CHANGED, 30,
+			notify_changes, backend);
 
 	thread = g_thread_create ((GThreadFunc) get_changed_events, E_CAL_BACKEND_EXCHANGE (backend), FALSE, &error);
 	if (!thread) {
@@ -1173,7 +1193,7 @@ modify_object_with_href (ECalBackendSync *backend, EDataCal *cal,
 
 	update_x_properties (E_CAL_BACKEND_EXCHANGE (cbexc), updated_ecomp);
 
-	last_modified = icaltime_from_timet (time (NULL), 0);
+	last_modified = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 	e_cal_component_set_last_modified (updated_ecomp, &last_modified);
 
 	if (e_cal_component_has_attachments (updated_ecomp)) {
@@ -1563,6 +1583,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 	icalproperty_method method;
 	icalcomponent *subcomp, *icalcomp;
 	ECalBackendSyncStatus status = GNOME_Evolution_Calendar_Success;
+	GError *error = NULL;
 
 	d(printf ("ecbexc_modify_object(%p, %p, %s)", backend, cal, calobj ? calobj : NULL));
 	cbexc =	E_CAL_BACKEND_EXCHANGE_CALENDAR (backend);
@@ -1582,6 +1603,20 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		return GNOME_Evolution_Calendar_InvalidObject;
 
 	icalcomp = icalparser_parse_string (calobj);
+
+	/* map time zones against system time zones and handle conflicting definitions */
+	if (icalcomp &&
+	    !e_cal_check_timezones (icalcomp,
+				    comps,
+				    e_cal_backend_exchange_lookup_timezone,
+				    cbex,
+				    &error)) {
+		g_warning ("checking timezones failed: %s", error->message);
+		icalcomponent_free (icalcomp);
+		g_clear_error (&error);
+		return GNOME_Evolution_Calendar_InvalidObject;
+	}
+
 	add_timezones_from_comp (E_CAL_BACKEND_EXCHANGE (backend), icalcomp);
 	icalcomponent_free (icalcomp);
 
@@ -1596,7 +1631,7 @@ receive_objects (ECalBackendSync *backend, EDataCal *cal,
 		e_cal_component_set_icalcomponent (comp, subcomp);
 
 		/*create time and last modified*/
-		current = icaltime_from_timet (time (NULL), 0);
+		current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 		e_cal_component_set_created (comp, &current);
 		e_cal_component_set_last_modified (comp, &current);
 
@@ -1977,10 +2012,11 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendExchange *cbex = (ECalBackendExchange *) backend;
 	ECalBackendSyncStatus retval = GNOME_Evolution_Calendar_Success;
 	ECalBackendExchangeBookingResult result;
-	ECalComponent *comp;
-	icalcomponent *top_level, *icalcomp, *tzcomp;
+	ECalComponent *comp = NULL;
+	icalcomponent *top_level = NULL, *icalcomp;
 	icalproperty *prop;
 	icalproperty_method method;
+	GError *error = NULL;
 
 	g_return_val_if_fail (E_IS_CAL_BACKEND_EXCHANGE (cbex),
 				GNOME_Evolution_Calendar_InvalidObject);
@@ -1993,6 +2029,20 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	*modified_calobj = NULL;
 
 	top_level = icalparser_parse_string (calobj);
+
+	/* map time zones against system time zones and handle conflicting definitions */
+	if (top_level &&
+	    !e_cal_check_timezones (top_level,
+				    NULL,
+				    e_cal_backend_exchange_lookup_timezone,
+				    cbex,
+				    &error)) {
+		g_warning ("checking timezones failed: %s", error->message);
+		g_clear_error (&error);
+		retval = FALSE;
+		goto cleanup;
+	}
+
 	icalcomp = icalcomponent_new_clone (icalcomponent_get_inner (top_level));
 
 	comp = e_cal_component_new ();
@@ -2008,13 +2058,7 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	}
 
 	/* traverse all timezones to add them to the backend */
-	tzcomp = icalcomponent_get_first_component (top_level,
-						    ICAL_VTIMEZONE_COMPONENT);
-	while (tzcomp) {
-		e_cal_backend_exchange_add_timezone (cbex, tzcomp);
-		tzcomp = icalcomponent_get_next_component (top_level,
-							   ICAL_VTIMEZONE_COMPONENT);
-	}
+	add_timezones_from_comp (cbex, top_level);
 
 	for (prop = icalcomponent_get_first_property (icalcomp, ICAL_ATTENDEE_PROPERTY);
 	     prop != NULL;
@@ -2083,8 +2127,12 @@ send_objects (ECalBackendSync *backend, EDataCal *cal,
 	*modified_calobj = g_strdup (e_cal_component_get_as_string (comp));
 
  cleanup:
-	icalcomponent_free (top_level);
-	g_object_unref (comp);
+	if (top_level) {
+		icalcomponent_free (top_level);
+	}
+	if (comp) {
+		g_object_unref (comp);
+	}
 
 	return retval;
 }
